/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.bizcustomer.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.PhoneUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.biz.modular.bizcustomer.entity.BizCustomer;
import vip.xiaonuo.biz.modular.bizcustomer.mapper.BizCustomerMapper;
import vip.xiaonuo.biz.modular.bizcustomer.param.BizCustomerAddParam;
import vip.xiaonuo.biz.modular.bizcustomer.param.BizCustomerEditParam;
import vip.xiaonuo.biz.modular.bizcustomer.param.BizCustomerIdParam;
import vip.xiaonuo.biz.modular.bizcustomer.param.BizCustomerPageParam;
import vip.xiaonuo.biz.modular.bizcustomer.service.BizCustomerService;

import java.util.List;

/**
 * 客户Service接口实现类
 *
 * @author fanzherong
 * @date  2025/03/21 11:44
 **/
@Service
public class BizCustomerServiceImpl extends ServiceImpl<BizCustomerMapper, BizCustomer> implements BizCustomerService {

    @Override
    public Page<BizCustomer> page(BizCustomerPageParam bizCustomerPageParam) {
        QueryWrapper<BizCustomer> queryWrapper = new QueryWrapper<BizCustomer>().checkSqlInjection();
        if(ObjectUtil.isNotEmpty(bizCustomerPageParam.getSearchKey())){
            queryWrapper.lambda().like(BizCustomer::getCustomerName,bizCustomerPageParam.getSearchKey());
        }
        queryWrapper.lambda().orderByDesc(BizCustomer::getCreateTime);
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(BizCustomerAddParam bizCustomerAddParam) {
        //校验手机号格式
        if(ObjectUtil.isNotEmpty(bizCustomerAddParam.getCustomerPhone())){
            if(!PhoneUtil.isMobile(bizCustomerAddParam.getCustomerPhone())) {
                throw new CommonException("手机号码：{}格式错误", bizCustomerAddParam.getCustomerPhone());
            }
        }
        //查询客户名称是否添加过
        long count = this.count(new QueryWrapper<BizCustomer>().lambda().eq(BizCustomer::getCustomerName, bizCustomerAddParam.getCustomerName()));
        if(count>0){
            throw new CommonException("客户名称已存在！");
        }
        BizCustomer bizCustomer = BeanUtil.toBean(bizCustomerAddParam, BizCustomer.class);
        this.save(bizCustomer);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(BizCustomerEditParam bizCustomerEditParam) {
        //校验手机号格式
        if(ObjectUtil.isNotEmpty(bizCustomerEditParam.getCustomerPhone())){
            if(!PhoneUtil.isMobile(bizCustomerEditParam.getCustomerPhone())) {
                throw new CommonException("手机号码：{}格式错误", bizCustomerEditParam.getCustomerPhone());
            }
        }
        BizCustomer bizCustomer = this.queryEntity(bizCustomerEditParam.getId());
        if(!StringUtils.equals(bizCustomer.getCustomerName(),bizCustomerEditParam.getCustomerName())){
            //查询客户名称是否添加过
            long count = this.count(new QueryWrapper<BizCustomer>().lambda().eq(BizCustomer::getCustomerName, bizCustomerEditParam.getCustomerName()));
            if(count>0){
                throw new CommonException("客户名称已存在！");
            }
        }
        BeanUtil.copyProperties(bizCustomerEditParam, bizCustomer);
        this.updateById(bizCustomer);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<BizCustomerIdParam> bizCustomerIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(bizCustomerIdParamList, BizCustomerIdParam::getId));
    }

    @Override
    public BizCustomer detail(BizCustomerIdParam bizCustomerIdParam) {
        return this.queryEntity(bizCustomerIdParam.getId());
    }

    @Override
    public BizCustomer queryEntity(String id) {
        BizCustomer bizCustomer = this.getById(id);
        if(ObjectUtil.isEmpty(bizCustomer)) {
            throw new CommonException("客户不存在，id值为：{}", id);
        }
        return bizCustomer;
    }
}
